<?php
namespace PME\Controllers\RestricoesContratuais;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class FrequenciaProdController {
    
  protected $entityManager;
  protected $newCode;
  protected $environment;
  protected $connection;
  
  public function __construct($entityManager, $connection, $newCode, $environment) {
  	$this->entityManager = $entityManager;
  	$this->newCode = $newCode;
  	$this->environment = $environment;
  	$this->connection = $connection;
  }
  
  public function returnCodFreqProd(DTO\Request $request, DTO\Response $response) {
    $sql = "SELECT MAX(NRFREQPRODMESPAE) CODIGO FROM FREQPRODMESPAE";

    $codigo = $this->entityManager->getConnection()->fetchAll($sql);
    $response->addDataSet(new DataSet("codigo", $codigo));
  }
  
  public function getMaxItFreqProd() {
    $sql = "SELECT MAX(NRITFREQPRODMESPAE) NRITFREQPRODMESPAE FROM ITFREQPRODMESPAE";

    $result = $this->entityManager->getConnection()->fetchAssoc($sql);
    return $result;
  }
  
  private function validaPorProd($param){
    // Valida se o produto já é cadastrado na restrição
		$sql = "SELECT I.NRITFREQPRODMESPAE, P.NMPRODUTO
                 FROM ITFREQPRODMESPAE I
                 INNER JOIN PRODUTO P ON P.CDPRODUTO = I.CDPRODUTO 
                WHERE I.CDPRODUTO = :CDPRODUTO
                  AND I.NRFREQPRODMESPAE = :NRFREQPRODMESPAE
                   AND I.NRORG = :NRORG";
        
    $paramSql = array(
        'CDPRODUTO' => $param['CDPRODUTO'],
        'NRFREQPRODMESPAE' => $param['NRFREQPRODMESPAE'],
        'NRORG' => $this->environment->getNrOrg()
    );
    
    $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
    if (count($resultado) > 0) {
    	return 'ERROR_PROD_CADASTRADO_VIG:' . $resultado[0]['NMPRODUTO'];
    }
    
    return true;
  }
  
  public function saveItFreqProd(DTO\Request $request, DTO\Response $response) {
      try {			
    			$params = $request->getParameters()["editedRows"];
                $this->entityManager->getConnection()->beginTransaction();
    			
    			foreach ($params as $param) {
    			    if ($param['__is_new'] === true) {
    			        $validacao = $this->validaPorProd($param);
					
        					if ($validacao !== true) {
        						$this->entityManager->getConnection()->rollBack();
        						$response->addDataset(new DataSet('mensagem', array($validacao)));
        						return;
        					};
    			      
        			    $nrItFrqProd = $this->getMaxItFreqProd();
        				
        				// Insere produto na frequencia de produto
        				$sql = "INSERT INTO ITFREQPRODMESPAE (NRITFREQPRODMESPAE, CDPRODUTO, NRMINFREQPRODMES, 
        				          NRMAXFREQPRODMES, NRMAXFREQPRODSEM, NRFREQPRODMESPAE, NRORG)
        				        VALUES (:NRITFREQPRODMESPAE, :CDPRODUTO, :NRMINFREQPRODMES, :NRMAXFREQPRODMES,
        				            :NRMAXFREQPRODSEM, :NRFREQPRODMESPAE, :NRORG)";
                        
                        $paramsIn = array(
                            'NRITFREQPRODMESPAE' => $nrItFrqProd['NRITFREQPRODMESPAE'] + 1,
                            'CDPRODUTO' => $param['CDPRODUTO'],
                            'NRMINFREQPRODMES' => $param['NRMINFREQPRODMES'],
                            'NRMAXFREQPRODMES' => $param['NRMAXFREQPRODMES'],
                            'NRMAXFREQPRODSEM' => $param['NRMAXFREQPRODSEM'],
                            'NRFREQPRODMESPAE' => $param['NRFREQPRODMESPAE'],
                            'NRORG' => $this->environment->getNrOrg()
                        );
                        
                        $this->entityManager->getConnection()->prepare($sql)->execute($paramsIn);
    			    } else {
    			        // Atualiza item de frequência existente
    			        $sql = "UPDATE ITFREQPRODMESPAE SET NRMINFREQPRODMES = :NRMINFREQPRODMES, NRMAXFREQPRODMES = :NRMAXFREQPRODMES,
    			                    NRMAXFREQPRODSEM = :NRMAXFREQPRODSEM
    			                 WHERE NRITFREQPRODMESPAE = :NRITFREQPRODMESPAE AND CDPRODUTO = :CDPRODUTO AND NRFREQPRODMESPAE = :NRFREQPRODMESPAE
    			                    AND NRORG = :NRORG";
                        
                        $paramsIn = array(
                            'NRITFREQPRODMESPAE' => $param['NRITFREQPRODMESPAE'],
                            'CDPRODUTO' => $param['CDPRODUTO'],
                            'NRMINFREQPRODMES' => $param['NRMINFREQPRODMES'],
                            'NRMAXFREQPRODMES' => $param['NRMAXFREQPRODMES'],
                            'NRMAXFREQPRODSEM' => $param['NRMAXFREQPRODSEM'],
                            'NRFREQPRODMESPAE' => $param['NRFREQPRODMESPAE'],
                            'NRORG' => $this->environment->getNrOrg()
                        );
                        
                        $this->entityManager->getConnection()->prepare($sql)->execute($paramsIn);
    			    }
    			}
			
            $this->entityManager->getConnection()->commit();
        }catch (\Exception $e) {
            $this->entityManager->getConnection()->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
  }
  
  public function getMaxItFreqProdGrPr() {
    $sql = "SELECT MAX(NRITFREQPRODGRPRMPAE) NRITFREQPRODGRPRMPAE FROM ITFREQPRODGRPRMPAE";

    $result = $this->entityManager->getConnection()->fetchAssoc($sql);
    return $result;
  }
  
  private function validaPorGrupoPrato($param){
    // Valida se o produto do grupo de prato já é cadastrado na restrição
		$sql = "SELECT I.NRITFREQPRODGRPRMPAE, G.NMGRUPOPRATOPAE
                 FROM ITFREQPRODGRPRMPAE I
                 INNER JOIN GRUPOPRATOPAE G ON G.NRGRUPOPRATOPAE = I.NRGRUPOPRATOPAE 
                WHERE I.NRGRUPOPRATOPAE = :NRGRUPOPRATOPAE
                  AND I.NRFREQPRODMESPAE = :NRFREQPRODMESPAE
                  AND I.CDPRODUTO = :CDPRODUTO
                   AND I.NRORG = :NRORG";
        
    $paramSql = array(
        'CDPRODUTO' => $param['CDPRODUTO'],
        'NRGRUPOPRATOPAE' => $param['NRGRUPOPRATOPAE'],
        'NRFREQPRODMESPAE' => $param['NRFREQPRODMESPAE'],
        'NRORG' => $this->environment->getNrOrg()
    );
    
    $resultado = $this->entityManager->getConnection()->fetchAll($sql, $paramSql);
    if (count($resultado) > 0) {
    	return 'ERROR_PROD_GRUP_CADASTRADO_VIG:' . $resultado[0]['NMGRUPOPRATOPAE'];
    }
    
    return true;
  }
  
  public function saveItFreqProdGrPr(DTO\Request $request, DTO\Response $response) {
      try {			
    			$params = $request->getParameters()["editedRows"];
                $this->entityManager->getConnection()->beginTransaction();
    			
    			foreach ($params as $param) {
    			    if ($param['__is_new'] === true) {
    			        $validacao = $this->validaPorGrupoPrato($param);
					
        					if ($validacao !== true) {
        						$this->entityManager->getConnection()->rollBack();
        						$response->addDataset(new DataSet('mensagem', array($validacao)));
        						return;
        					};
    			      
        			    $nrItFrqProd = $this->getMaxItFreqProdGrPr();
        				
        				// Insere grupo de prato na frequencia de produto
        				$sql = "INSERT INTO ITFREQPRODGRPRMPAE (NRITFREQPRODGRPRMPAE, NRGRUPOPRATOPAE, CDPRODUTO, NRMINFREQGRPRATMES, 
        				          NRMAXFREQGRPRATMES, NRMAXFREQGRPRATSEM, NRFREQPRODMESPAE, NRORG)
        				        VALUES (:NRITFREQPRODGRPRMPAE, :NRGRUPOPRATOPAE, :CDPRODUTO, :NRMINFREQGRPRATMES, :NRMAXFREQGRPRATMES,
        				            :NRMAXFREQGRPRATSEM, :NRFREQPRODMESPAE, :NRORG)";
                        
                        $paramsIn = array(
                            'NRITFREQPRODGRPRMPAE' => $nrItFrqProd['NRITFREQPRODGRPRMPAE'] + 1,
                            'NRGRUPOPRATOPAE' => $param['NRGRUPOPRATOPAE'],
                            'NRMINFREQGRPRATMES' => $param['NRMINFREQGRPRATMES'],
                            'NRMAXFREQGRPRATMES' => $param['NRMAXFREQGRPRATMES'],
                            'NRMAXFREQGRPRATSEM' => $param['NRMAXFREQGRPRATSEM'],
                            'NRFREQPRODMESPAE' => $param['NRFREQPRODMESPAE'],
                            'CDPRODUTO' => $param['CDPRODUTO'],
                            'NRORG' => $this->environment->getNrOrg()
                        );
                        
                    $this->entityManager->getConnection()->prepare($sql)->execute($paramsIn);
    			    } else {
    			        // Atualiza item de frequência existente
    			        $sql = "UPDATE ITFREQPRODGRPRMPAE SET NRMINFREQGRPRATMES = :NRMINFREQGRPRATMES, NRMAXFREQGRPRATMES = :NRMAXFREQGRPRATMES,
    			                    NRMAXFREQGRPRATSEM = :NRMAXFREQGRPRATSEM
    			                 WHERE NRITFREQPRODGRPRMPAE = :NRITFREQPRODGRPRMPAE AND NRGRUPOPRATOPAE = :NRGRUPOPRATOPAE AND NRFREQPRODMESPAE = :NRFREQPRODMESPAE
    			                    AND CDPRODUTO = :CDPRODUTO AND NRORG = :NRORG";
                        
                        $paramsIn = array(
                            'NRITFREQPRODGRPRMPAE' => $param['NRITFREQPRODGRPRMPAE'],
                            'NRGRUPOPRATOPAE' => $param['NRGRUPOPRATOPAE'],
                            'CDPRODUTO' => $param['CDPRODUTO'],
                            'NRMINFREQGRPRATMES' => $param['NRMINFREQGRPRATMES'],
                            'NRMAXFREQGRPRATMES' => $param['NRMAXFREQGRPRATMES'],
                            'NRMAXFREQGRPRATSEM' => $param['NRMAXFREQGRPRATSEM'],
                            'NRFREQPRODMESPAE' => $param['NRFREQPRODMESPAE'],
                            'NRORG' => $this->environment->getNrOrg()
                        );
                        
                        $this->entityManager->getConnection()->prepare($sql)->execute($paramsIn);
    			    }
    			}
			
            $this->entityManager->getConnection()->commit();
        }catch (\Exception $e) {
            $this->entityManager->getConnection()->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
  }
  
  public function excluiFreqProdGrPr(DTO\Request $request, DTO\Response $response) {
		try {		
			$params = $request->getParameters()["checkedRows"];
            $this->entityManager->getConnection()->beginTransaction();
			
			foreach ($params as $param) {
				$sql = "DELETE FROM ITFREQPRODGRPRMPAE 
                         WHERE NRGRUPOPRATOPAE = :NRGRUPOPRATOPAE 
                           AND NRFREQPRODMESPAE = :NRFREQPRODMESPAE
                            AND NRORG = :NRORG";
                
                $params = array(
                    'NRFREQPRODMESPAE' => $param['NRFREQPRODMESPAE'],
                    'NRGRUPOPRATOPAE' => $param['NRGRUPOPRATOPAE'],
                    'NRORG' => $this->environment->getNrOrg()
                );
                
                $this->connection->prepare($sql)->execute($params);
                
			}
			
            $this->entityManager->getConnection()->commit();
        }catch (\Exception $e) {
           $this->entityManager->getConnection()->rollBack(); 
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

}