import {
  ModalService, Modal, Form, Select, Grid, IButtonEvent, GridEditable, IComponentRender, Button, DialogService,
} from '@zeedhi/common';
import {
  Loader, Metadata, IEventParam, IDictionary, Http,
} from '@zeedhi/core';

export class Pla01313ReplicacaoReceitasServicoController {
  private utilities = Loader.getInstance('Utilities');
  
  public modalReplicacaoReceitasServico!: Modal;
  
  public modalEmpty!: Modal;
  
  public prato = Loader.getInstance('Pla01300PratoController');
  
  public appController = Loader.getInstance('AppController');
  
  public gridEditableController = Loader.getInstance('GridEditableController');
  
  private async createModalReplicacaoReceitasServico() {
    this.modalReplicacaoReceitasServico = await ModalService.createFromJson(
      'planejamento/prato/modals/pla01313_replicacao_receitas_servico', true,
    );
  }

  public closeModalReplicacaoReceitasServico() {
    this.clearModalFields();
    this.modalReplicacaoReceitasServico.hide();
    this.modalReplicacaoReceitasServico.destroy();
    this.modalReplicacaoReceitasServico = this.modalEmpty;
  }
  
  public async openModalReplicacaoReceitasServico() {
    if (!this.modalReplicacaoReceitasServico) {
      this.createModalReplicacaoReceitasServico().then(() => {
        this.modalReplicacaoReceitasServico.show();
        this.clearModalFields();
      });
    } else {
      await this.modalReplicacaoReceitasServico.show();
      this.clearModalFields();
    }
  }
  
  public fullscreen({ component }: IButtonEvent) {
    this.modalReplicacaoReceitasServico.fullscreen = !this.modalReplicacaoReceitasServico.fullscreen;
    component.iconName = (this.modalReplicacaoReceitasServico.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }
  
  public inicContainerOrigem() {
    const project = localStorage.getItem('PRODUCT_ID');
    
    const tpReceitaCard = Metadata.getInstance<Select>('TPRECEITA');
    const tpReceita = Metadata.getInstance<Select>('TPRECEITA1');
    const cardapio = Metadata.getInstance<Select>('TIPOCARD');

    if (project === '4875') {
      tpReceitaCard.isVisible = true;
      tpReceita.isVisible = false;
    } else {
      tpReceitaCard.isVisible = false;
      tpReceita.isVisible = true;
      cardapio.isVisible = false;
    }
  }
  
  public setFilterService({ component }: IEventParam<Select>) {
    const form: Form = Metadata.getInstance('formAcoesPrato');
    const row = form.value;
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: row.UNIDADE,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: localStorage.getItem('PRODUCT_ID') === '4875' ? '' : 'N',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public setFilterServicos() {
    const form: Form = Metadata.getInstance('formAcoesPrato2');
    const row = form.value;
    const grid: Grid = Metadata.getInstance('gridServico');
  
    if (form.validate()) {
      const conditions = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: row.UNIDADEDEST !== undefined ? row.UNIDADEDEST : 'T',
        },
        {
          name: 'CDGRUPSERV',
          operator: '=',
          value: row.GRUPOSERVICO !== null ? row.GRUPOSERVICO : 'T',
        },
        {
          name: 'IDTIPOSERV',
          operator: '=',
          value: localStorage.getItem('PRODUCT_ID') === '4875' ? '' : 'N',
        },
      ];
      grid.datasource.filter = conditions;
      grid.datasource.get();
    } else {
      grid.datasource.data = [];
      grid.datasource.total = 0;
      grid.datasource.filter = [];
      grid.selectedRows = [];
    }
  }
  
  public saveReplicacaoServicos() {
    const form = Metadata.getInstance<Form>('formAcoesPrato2');
    const grid: GridEditable = Metadata.getInstance('gridServico');
    const editedRows = grid.getEditedRows() as IDictionary;

    if (form.validate()) {
      if (editedRows.length > 0) {
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        this.atualizarGridReplicacaoServicos();
      } else {
        this.gridEditableController.cancelEditing({ component: grid });
        this.utilities.showAlert('alert', 'pme::nenhumRegEditado', 'error');
      }
    }
  }
  
  public atualizarGridReplicacaoServicos() {
    const grid: GridEditable = Metadata.getInstance('gridServico');
    const editedRows = grid.getEditedRows() as IDictionary;
    editedRows.forEach((editedRow: IDictionary) => {
      const foundInd = grid.datasource.data.findIndex((service: IDictionary) => service.CDSERVFILI === editedRow.CDSERVFILI);
      grid.datasource.data[foundInd].PERCENTUAL = editedRow.PERCENTUAL;
    });
    grid.editing = false;
    const toolbarSlot = grid.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = !e.disabled;
      if (e.name === 'saveButton') e.disabled = !e.disabled;
    });
  }
  
  public confirmarAcao({ component }: IEventParam<Button>) {
    const gridServico: GridEditable = Metadata.getInstance('gridServico');
    if (gridServico.editing === true) {
      DialogService.show({
        name: 'alteracoesSemSalvar',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::alteracoesSemSalvar',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => { this.saveReplicacaoServicos(); this.callReplicacaoReceitaPorServico(component); } },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: {
              click: () => {
                this.gridEditableController.cancelEditing({ component: gridServico });
                this.callReplicacaoReceitaPorServico(component);
              },
            },
          },
        ],
      });
    } else {
      this.callReplicacaoReceitaPorServico(component);
    }
  }
  
  public callReplicacaoReceitaPorServico(component: any) {
    DialogService.hide();
    const form: Form = Metadata.getInstance('formAcoesPrato');
    const row = form.value;
    const gridPrato: Grid = Metadata.getInstance('gridPrato');
    const rowsGridPrato = gridPrato.selectedRows;
    const gridServico: GridEditable = Metadata.getInstance('gridServico');
    const rowsGridServico = gridServico.selectedRows;
    
    if (row.TPRECEITA === null) row.TPRECEITA = row.TPRECEITA1;
    
    if (form.validate()) {
      if (rowsGridPrato.length > 0) {
        if (rowsGridServico.length > 0) {
          this.utilities.activeLoadingButton(component);
          row.CDFILCENTPME = this.appController.getCdCentral();
          const params = {
            ORIGEM: row,
            PRATOS: rowsGridPrato,
            SERVICOS: rowsGridServico,
          };
          Http.post('/replicaReceitaPorServico', { requestType: 'Row', row: params }).then((response: IDictionary) => {
            if (response.data) {
              if (response.data.dataset) {
                if (response.data.dataset.RETORNO[0]) {
                  this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
                  gridServico.cancelEditedRows();
                }
              } else this.utilities.showAlert('alert', `pme::${response.data.error}`, 'error');
            }
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `pme::${error}`, 'error');
          }).finally(() => {
            this.utilities.disableLoadingButton(component);
            gridServico.selectedRows = [];
            gridPrato.selectedRows = [];
            // DialogService.hide();
          });
        } else this.utilities.showAlert('Alert', 'pme::aoMenosUmServicoSelecionado', 'error');
      } else this.utilities.showAlert('Alert', 'pme::aoMenosUmPratoSelecionado', 'error');
    }
  }
  
  public clearModalFields() {
    const fields = [
      'TIPOCARD', 'UNIDADE', 'UNIDADEDEST', 'GRUPOSERVICO', 'SERVICO', 'PRATOINICIAL', 'PRATOFINAL',
    ];
    fields.forEach((fieldName) => {
      if (document.getElementsByName(fieldName).length > 0) Metadata.getInstance<Select>(fieldName).setValue('');
    });
    
    if (document.getElementsByName('TPRECEITA').length > 0) Metadata.getInstance<Select>('TPRECEITA').setValue('P');
    if (document.getElementsByName('TPRECEITA1').length > 0) Metadata.getInstance<Select>('TPRECEITA1').setValue('P');
    
    if (document.getElementsByName('gridPrato').length > 0) {
      const gridPrato: Grid = Metadata.getInstance('gridPrato');
      gridPrato.datasource.data = [];
      gridPrato.datasource.total = 0;
      gridPrato.datasource.filter = [];
      gridPrato.selectedRows = [];
    }
    
    if (document.getElementsByName('gridServico').length > 0) {
      const gridPrato: Grid = Metadata.getInstance('gridServico');
      gridPrato.datasource.data = [];
      gridPrato.datasource.total = 0;
      gridPrato.datasource.filter = [];
      gridPrato.selectedRows = [];
    }
  }
}
